from django.db.models import Manager


class TaskManager(Manager):
    def parents(self, task):
        """
        List all tasks that a task depends on, directly or not.
        A PostgreSQL recursive query is used for better performances.
        """
        return self.raw(
            """
            with recursive parents(id) as (
                select to_task_id
                    from ponos_task_parents j
                    where j.from_task_id = %s
                union all
                    select j.to_task_id
                    from ponos_task_parents as j, parents
                    where j.from_task_id = parents.id
            )
            select distinct * from parents
            """,
            [task.id],
        )
