from django.core.management.base import BaseCommand

from arkindex.process.models import Process, ProcessMode
from arkindex.project.argparse import UserArgument, WorkerVersionArgument


class Command(BaseCommand):
    help = "Create a WorkerRun on a user's local process"

    def add_arguments(self, parser):
        parser.add_argument(
            '--user',
            help='Email or ID of the user',
            required=True,
            type=UserArgument(),
        )
        parser.add_argument(
            '--worker-version-id',
            help='ID of the WorkerVersion',
            required=True,
            type=WorkerVersionArgument(),
            dest='worker_version',
        )

    def handle(self, user, worker_version, *args, **options):
        process, created = Process.objects.get_or_create(
            creator=user,
            mode=ProcessMode.Local,
        )

        if created:
            self.stdout.write(f'Created local process {process.id}')
        else:
            self.stdout.write(f'Using existing local process {process.id}')

        worker_run, created = process.worker_runs.get_or_create(
            version=worker_version,
            defaults={'parents': []},
        )

        if created:
            self.stdout.write(self.style.SUCCESS(f'Created WorkerRun {worker_run.id} ({worker_run.summary})'))
        else:
            self.stdout.write(self.style.WARNING(f'A WorkerRun for this worker version already exists: {worker_run.id} ({worker_run.summary})'))
