# Generated by Django 4.1.7 on 2023-06-01 09:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ponos', '0002_task_process'),
        ('process', '0004_process_farm_started_finished'),
        ('training', '0002_initial'),
    ]

    operations = [
        migrations.RunSQL(
            [
                # In some rare cases, the workflow creation date is set a few milliseconds
                # before the process creation date, breaking the process_started_after_created constraint
                """
                update ponos_workflow w
                set created = data.created
                from (
                    select w.id, p.created
                    from process_process p
                    inner join ponos_workflow w on (w.id=p.workflow_id)
                    where w.created < p.created
                ) data
                where data.id=w.id;
                """,
                # This query cannot be done with QuerySet.update, because Django does not allow joining during an update.
                # This is why we use RunSQL here and not RunPython.
                """
                UPDATE process_process process
                SET started = workflow.created, finished = workflow.finished, farm_id = workflow.farm_id
                FROM ponos_workflow workflow
                WHERE process.workflow_id = workflow.id
                """,
                """
                UPDATE ponos_task task
                SET process_id = process.id
                FROM process_process process
                WHERE process.workflow_id = task.workflow_id
                """,

                # We might have some tasks that are in workflows that do not have any processes.
                # We have no choice but to delete those tasks, so we also set foreign keys to NULL or delete related rows.

                # Unlink the Docker image artifacts from worker versions for the tasks we are about to delete
                """
                UPDATE process_workerversion version
                SET state = 'error', docker_image_id = NULL
                FROM ponos_artifact artifact, ponos_task task
                WHERE version.docker_image_id = artifact.id
                AND artifact.task_id = task.id
                AND task.process_id IS NULL
                """,
                # Unlink the Docker image artifacts from tasks for the tasks we are about to delete
                """
                UPDATE ponos_task task
                SET image_artifact_id = NULL
                FROM ponos_artifact artifact, ponos_task task2
                WHERE task.image_artifact_id = artifact.id
                AND artifact.task_id = task2.id
                AND task2.process_id IS NULL
                """,
                # Remove the artifacts of the tasks we are about to delete
                """
                DELETE FROM ponos_artifact artifact
                USING ponos_task task
                WHERE artifact.task_id = task.id
                AND task.process_id IS NULL
                """,
                # Unlink the tasks we are about to delete from datasets
                """
                UPDATE training_dataset dataset
                SET task_id = NULL
                FROM ponos_task task
                WHERE dataset.task_id = task.id
                AND task.process_id IS NULL
                """,
                # Remove task parents
                """
                DELETE FROM ponos_task_parents parents
                USING ponos_task task
                WHERE parents.from_task_id = task.id
                AND task.process_id IS NULL
                """,
                """
                DELETE FROM ponos_task_parents parents
                USING ponos_task task
                WHERE parents.to_task_id = task.id
                AND task.process_id IS NULL
                """,
                # Remove the tasks
                """
                DELETE FROM ponos_task
                WHERE process_id IS NULL
                """,
            ],
            reverse_sql=[
                # When reverting, we need to recreate workflows for any process that has task, then reassign the tasks to the workflows.
                # For convenience, we will use the process IDs as the new workflow IDs, since nothing forbids that.
                """
                INSERT INTO ponos_workflow (id, farm_id, created, updated, finished)
                SELECT id, farm_id, started, updated, finished
                FROM process_process process
                WHERE EXISTS (
                    SELECT 1
                    FROM ponos_task
                    WHERE process_id = process.id
                )
                """,
                """
                UPDATE process_process process
                SET workflow_id = id
                WHERE EXISTS (
                    SELECT 1
                    FROM ponos_task
                    WHERE process_id = process.id
                )
                """,
                """
                UPDATE ponos_task
                SET workflow_id = process_id
                """,
            ],
            # manage.py squashmigrations is allowed to remove this data migration
            elidable=True,
        ),
    ]
